VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "CRegistry"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'********************************************
'* CRegistry.cls                            *
'*                                          *
'* Class for reading, writing or checking   *
'* the registry.                            *
'*                                          *
'* You can use this code for free at your   *
'* own risk; the author doesn't correspond  *
'* to any damages created by this code.     *
'*                                          *
'* (C) 2004 by Claus von der Burchard       *
'* for AVB Messenger                        *
'* contact: claus@cvdb.de                   *
'*                                          *
'* Last Change: 20-Apr-2004                 *
'********************************************

Option Explicit


'WIN32 APIs
Private Declare Function RegOpenKeyEx Lib "advapi32.dll" Alias "RegOpenKeyExA" ( _
  ByVal hKey As Long, _
  ByVal lpSubKey As String, _
  ByVal ulOptions As Long, _
  ByVal samDesired As Long, _
  phkResult As Long _
) As Long
        
Private Declare Function RegCloseKey Lib "advapi32.dll" ( _
  ByVal hKey As Long _
) As Long
        
Private Declare Function RegQueryValueEx Lib "advapi32.dll" Alias "RegQueryValueExA" ( _
  ByVal hKey As Long, _
  ByVal lpValueName As String, _
  ByVal lpReserved As Long, _
  lpType As Long, _
  lpData As Any, _
  lpcbData As Any _
) As Long
        
Private Declare Function RegCreateKeyEx Lib "advapi32.dll" Alias "RegCreateKeyExA" ( _
  ByVal hKey As Long, _
  ByVal lpSubKey As String, _
  ByVal Reserved As Long, _
  ByVal lpClass As String, _
  ByVal dwOptions As Long, _
  ByVal samDesired As Long, _
  ByVal lpSecurityAttributes As Any, _
  phkResult As Long, _
  lpdwDisposition As Long _
) As Long
        
Private Declare Function RegFlushKey Lib "advapi32.dll" ( _
  ByVal hKey As Long _
) As Long
        
Private Declare Function RegSetValueEx Lib "advapi32.dll" Alias "RegSetValueExA" ( _
  ByVal hKey As Long, _
  ByVal lpValueName As String, _
  ByVal Reserved As Long, _
  ByVal dwType As Long, _
  lpData As Long, _
  ByVal cbData As Long _
) As Long
        
Private Declare Function RegSetValueEx_Str Lib "advapi32.dll" Alias "RegSetValueExA" ( _
  ByVal hKey As Long, _
  ByVal lpValueName As String, _
  ByVal Reserved As Long, _
  ByVal dwType As Long, _
  ByVal lpData As String, _
  ByVal cbData As Long _
) As Long

Private Declare Function RegDeleteKey Lib "advapi32.dll" Alias "RegDeleteKeyA" ( _
  ByVal hKey As Long, _
  ByVal lpSubKey As String _
) As Long
        
Private Declare Function RegDeleteValue Lib "advapi32.dll" Alias "RegDeleteValueA" ( _
  ByVal hKey As Long, _
  ByVal lpValueName As String _
) As Long

'Constants
Private Const KEY_QUERY_VALUE           As Long = &H1
Private Const KEY_SET_VALUE             As Long = &H2
Private Const KEY_CREATE_SUB_KEY        As Long = &H4
Private Const KEY_ENUMERATE_SUB_KEYS    As Long = &H8
Private Const KEY_NOTIFY                As Long = &H10
Private Const KEY_CREATE_LINK           As Long = &H20
Private Const KEY_READ                  As Long = KEY_QUERY_VALUE Or _
                                                  KEY_ENUMERATE_SUB_KEYS Or _
                                                  KEY_NOTIFY
Private Const KEY_ALL_ACCESS            As Long = KEY_QUERY_VALUE Or _
                                                  KEY_SET_VALUE Or _
                                                  KEY_CREATE_SUB_KEY Or _
                                                  KEY_ENUMERATE_SUB_KEYS Or _
                                                  KEY_NOTIFY Or _
                                                  KEY_CREATE_LINK

Private Const ERROR_SUCCESS             As Long = 0&

Private Const REG_NONE                  As Long = 0
Private Const REG_SZ                    As Long = 1
Private Const REG_EXPAND_SZ             As Long = 2
Private Const REG_BINARY                As Long = 3
Private Const REG_DWORD                 As Long = 4
Private Const REG_DWORD_LITTLE_ENDIAN   As Long = 4
Private Const REG_DWORD_BIG_ENDIAN      As Long = 5
Private Const REG_LINK                  As Long = 6
Private Const REG_MULTI_SZ              As Long = 7

Private Const REG_OPTION_NON_VOLATILE   As Long = &H0

'Enums
Public Enum RootClasses
    HKEY_CLASSES_ROOT = &H80000000
    HKEY_CURRENT_USER = &H80000001
    HKEY_LOCAL_MACHINE = &H80000002
    HKEY_USERS = &H80000003
    HKEY_PERFORMANCE_DATA = &H80000004
    HKEY_CURRENT_CONFIG = &H80000005
    HKEY_DYN_DATA = &H80000006
End Enum

Public Enum RegKeyCreateValues
    regKeyCreateError = 0
    regKeyCreateOK = 1
    regKeyCreateAlreadyExists = 2
End Enum

Public Enum RegKeyExistsValues
    regExists = 0
    regNotExists = 1
End Enum

Public Enum RegValueSetValues
    regValueSetOK = 0
    regValueSetError = 1
End Enum

Public Enum RegValueGetValues
    regValueGetOK = 0
    regValueGetError = 1
End Enum

Public Enum RegFieldDeleteValues
    regFieldDeleteOK = 0
    regFieldDeleteError = 1
End Enum

Public Enum RegKeyDeleteValues
    regKeyDeleteOK = 0
    regKeyDeleteError = 1
End Enum
'---------------------------------------------------------------

Public Function RegKeyExist(Root As RootClasses, Key As String) As RegKeyExistsValues
    'Prfen ob ein Schlssel existiert
    Dim Result      As Long, _
        hKey        As Long
        
    Result = RegOpenKeyEx(Root, Key, 0, KEY_READ, hKey)
    If Result = ERROR_SUCCESS Then Call RegCloseKey(hKey)
    RegKeyExist = Result
End Function

Public Function RegKeyCreate(Root As RootClasses, Newkey As String) As RegKeyCreateValues
    'Neuen Schlssel erstellen
    Dim Result      As Long, _
        hKey        As Long, _
        Back        As Long
    
    Result = RegCreateKeyEx(Root, Newkey, 0, vbNullString, _
                            REG_OPTION_NON_VOLATILE, _
                            KEY_ALL_ACCESS, 0&, hKey, Back)
    
    If Result = ERROR_SUCCESS Then
        Result = RegFlushKey(hKey)
        If Result = ERROR_SUCCESS Then Call RegCloseKey(hKey)
        RegKeyCreate = Back
    End If
End Function

Public Function RegKeyDelete(Root As RootClasses, Key As String) As RegKeyDeleteValues
    'Schlssel lschen
    RegKeyDelete = RegDeleteKey(Root, Key)
End Function

Public Function RegFieldDelete(Root As RootClasses, Key As String, Field As String) As RegFieldDeleteValues
    'Feld lschen
    Dim Result      As Long, _
        hKey        As Long
        
    Result = RegOpenKeyEx(Root, Key, 0, KEY_ALL_ACCESS, hKey)
    If Result = ERROR_SUCCESS Then
        Result = RegDeleteValue(hKey, Field)
        Result = RegCloseKey(hKey)
    End If
    RegFieldDelete = Result
End Function

'FIXIT: 'Value' mit einem Datentyp mit frher Bindung deklarieren                          FixIT90210ae-R1672-R1B8ZE
Public Function RegValueSet(Root As RootClasses, Key As String, Field As String, Value As Variant) As RegValueSetValues
    'Wert in ein Feld der Registry schreiben
    Dim Result      As Long, _
        hKey        As Long, _
        l           As Long, _
        s           As String

    Result = RegOpenKeyEx(Root, Key, 0, KEY_ALL_ACCESS, hKey)
    If Result <> ERROR_SUCCESS Then
        Call RegKeyCreate(Root, Key)
        Result = RegOpenKeyEx(Root, Key, 0, KEY_ALL_ACCESS, hKey)
    End If
    
    If Result = ERROR_SUCCESS Then
        Select Case VarType(Value)
          Case vbInteger, vbLong
            l = CLng(Value)
            Result = RegSetValueEx(hKey, Field, 0, REG_DWORD, l, 4)
          Case vbString
            s = CStr(Value)
            If s = "" Then
                Result = RegSetValueEx_Str(hKey, Field, 0, REG_SZ, vbNullString, 0)
            Else
                Result = RegSetValueEx_Str(hKey, Field, 0, REG_SZ, s, Len(s) + 1)
            End If
        End Select
        Result = RegCloseKey(hKey)
    End If
    
    RegValueSet = Result
End Function

'FIXIT: 'Value' mit einem Datentyp mit frher Bindung deklarieren                          FixIT90210ae-R1672-R1B8ZE
Public Function RegValueGet(Root As RootClasses, Key As String, Field As String, Value As Variant) As RegValueGetValues
    'Wert aus einem Feld der Registry auslesen
    Dim Result      As Long, _
        hKey        As Long, _
        dwType      As Long, _
        Lng         As Long, _
        l           As Long, _
        Buffer      As String

    Result = RegOpenKeyEx(Root, Key, 0, KEY_READ, hKey)
    If Result = ERROR_SUCCESS Then
        Result = RegQueryValueEx(hKey, Field, 0&, dwType, ByVal 0&, l)
        If Result = ERROR_SUCCESS Then
          Select Case dwType
            Case REG_SZ
              Buffer = Space$(l - 1)
              Result = RegQueryValueEx(hKey, Field, 0&, _
                                       dwType, ByVal Buffer, l)
              If Result = ERROR_SUCCESS Then Value = Buffer
            Case REG_DWORD
              Result = RegQueryValueEx(hKey, Field, 0&, dwType, Lng, l)
              If Result = ERROR_SUCCESS Then Value = Lng
          End Select
        End If
    End If
    
    If InStrRev(Value, Chr$(0)) > 0 Then
        Value = Mid$(Value, 1, InStrRev(Value, Chr$(0)) - 1)
    End If
    
    If Result = ERROR_SUCCESS Then Result = RegCloseKey(hKey)
    RegValueGet = Result
End Function


