VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "CKeyExchange"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'********************************************
'* CCrypt.cls                               *
'*                                          *
'* Class for Diffie Hellmann Key Exchange   *
'* and creating random numbers and primes   *
'*                                          *
'* You can use this code for free at your   *
'* own risk; the author doesn't correspond  *
'* to any damages created by this code.     *
'*                                          *
'* (C) 2003 by Claus von der Burchard       *
'* for AVB Messenger                        *
'* contact: claus@cvdb.de                   *
'*                                          *
'* Last Change: 20-Apr-2004                 *
'********************************************

Option Explicit
Private Declare Function ASM_cdLong _
    Lib "user32" Alias "CallWindowProcA" _
   (ByRef asm As Long, _
    ByVal PA1 As Long, _
    ByVal PA2 As Long, _
    ByVal PA3 As Long, _
    ByVal PA4 As Long) As Long

Private Sub Class_Initialize()
    Randomize
End Sub


'PowerMod: Fast function to calculate (a ^ b) Mod c
Function PowerMod(Basis As Long, Exponent As Long, Modul As Long) As Long
    Static asm(8) As Long

    If asm(0) = 0 Then
        'source by Udo Schmidt
        asm(0) = &H748B5756:  asm(1) = &H4C8B0C24:  asm(2) = &H7C8B1024
        asm(3) = &H1B81424:   asm(4) = &HF7000000:  asm(5) = &H8BF7F7E6
        asm(6) = &HF77549C2:  asm(7) = &H10C25E5F:  asm(8) = &H0
    End If

    PowerMod = ASM_cdLong(asm(0), Basis, Exponent, Modul, 0)
End Function

'Basefunction to create a random number
Public Function RandomNumber(Optional Min As Single = 1, Optional Max As Single = 100) As Single
    Dim k1 As Single, k2 As Single
    k1 = Fix((Rnd * (50 - 19)) + 20)
    k2 = Fix((Rnd * (120 - 29)) + 30)
    RandomNumber = Fix((Rnd * (Max - Min + 1)) + Min) + (k1 Mod k2) Mod Fix(Rnd * (Max - Min + 1) + Min)
    If RandomNumber = 0 Then RandomNumber = Fix(Min)
    If RandomNumber > Max Then RandomNumber = RandomNumber(Min, Max)
End Function

'Uses RandomNumber to create a random number
'Counts the random number so long up till it is a prime
'using ASM to become faster
Public Function RandomPrime(Optional Min As Single = 1, Optional BaseMax As Single = 100) As Long
    Dim i           As Long
    Static asm2(10) As Long

    If asm2(0) = 0 Then
        asm2(0) = &H4244C8B:   asm2(1) = &H4901C980:  asm2(2) = &H2C18349
        asm2(3) = &H1BB:       asm2(4) = &H2C38300:   asm2(5) = &HD233C18B
        asm2(6) = &HD20BF3F7:  asm2(7) = &HC38BEB74:  asm2(8) = &HC13BE3F7
        asm2(9) = &HC18BEB72:  asm2(10) = &H10C2
    End If
    
    i = Fix(RandomNumber(Min, BaseMax))
    RandomPrime = ASM_cdLong(asm2(0), i, 0, 0, 0)
End Function

Public Function Create_p() As Single
    'p is a prime; p will be set bet between 6000000 and 9000000
    Create_p = RandomPrime(6000000, 9000000)
End Function

Public Function Create_g() As Single
    'g is any number; p will be set bet between 6000000 and 9000000
    Create_g = RandomNumber(6000000, 9000000)
End Function

'Function for the key exchange:
'calculates alpha = (g ^ a) Mod p
Public Function Create_alpha(G As Single, a As Single, p As Single) As Single
    Create_alpha = CSng(PowerMod(CCur(G), CCur(a), CCur(p)))
End Function

'Function for the key exchange:
'calculates k = (beta ^ a) Mod p
Public Function Create_k(beta As Single, a As Single, p As Single) As Single
    Create_k = CSng(PowerMod(CCur(beta), CCur(a), CCur(p)))
End Function

'CreateKey creates the key used for further coding
'The default length is 64 chars
Public Function CreateKey(Optional Chars As Integer = 64) As String
    Dim i As Long
    For i = 1 To Chars
        CreateKey = CreateKey & Chr$(RandomNumber(1, 255))
    Next
End Function

'Code en-/decodes the key calculated with CreateKey by using k as key
Public Function Code(Text As String, k As Single) As String
    Dim i As Long
    Dim k1 As Long, k2 As Long
    k1 = CSng(Left$(CStr(k), 3)) Mod 255
    k2 = CSng(Mid$(CStr(k), 3)) Mod 255
    For i = 1 To Len(Text) Step 2
        Code = Code & Chr$(Asc(Mid$(Text, i, 1)) Xor k1)
        If i + 1 <= Len(Text) Then Code = Code & Chr$(Asc(Mid$(Text, i + 1, 1)) Xor k2)
    Next
End Function


