VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "CStyleSheet"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'----------------------------------------------------
' Syntax definition of style sheet:
'
' <Stylesheet>  ::= (<Block> | <Comment>)+
' <Comment>     ::= CSTART ANY* CEND
' <Block>       ::= ID OBR <Defs> CBR
' <Defs>        ::= (<Def> DELIM | <Comment>)*
' <Def>         ::= <LHS> DEF <RHS>
' <LHS>         ::= ID
' <RHS>         ::= ID
'               |   NUM
'               |   STRING
'----------------------------------------------------

Private m_Error     As String
Private m_Scanner   As CStyleSheetScanner
Private m_Device    As CFormatVB
Private CurrentStyle As CStyleProps
'----------------------------------------------------

Public Sub SetOutput(Device As CFormatVB)
    Set m_Device = Device
End Sub

Public Sub Parse(Data As String)
    Set m_Scanner = New CStyleSheetScanner
    Call m_Scanner.StartScan(Data)
    m_Error = ""
    
    Call Tok_StyleSheet
End Sub
'----------------------------------------------------

Private Function Tok_StyleSheet() As Boolean
    If Not Tok_Block() Then
        Exit Function
    End If
    
    Do While Tok_Block()
    Loop
    
    Tok_StyleSheet = True
End Function

Private Function Tok_Block() As Boolean
    Select Case m_Scanner.GetNextToken()
        Case ID
        Case EOF
            Exit Function
        Case Else
            m_Error = "Expected ID"
            Exit Function
    End Select
    
    Select Case LCase$(TokenVal)
        Case "default"
            Set CurrentStyle = m_Device.Style_Normal
        Case "keywords"
            Set CurrentStyle = m_Device.Style_Keywords
        Case "comments"
            Set CurrentStyle = m_Device.Style_Comments
        Case "strings"
            Set CurrentStyle = m_Device.Style_String
        Case Else
            m_Error = "Unknown identifier"
    End Select
    
    If Not Match(OBR) Then
        m_Error = "Expected '{'"
        Exit Function
    End If
    
    If Not Tok_Defs() Then _
        Exit Function
    
    If Not Match(CBR) Then
        m_Error = "Expected '}'"
        Exit Function
    End If
    
    Tok_Block = True
End Function

Private Function Tok_Defs() As Boolean
    Do While Tok_Def()
        If Not Match(DELIM) Then
            m_Error = "Expected ';'"
            Exit Function
        End If
    Loop
    
    If LenB(m_Error) Then _
        Exit Function
    
    Call Rebound
    
    Tok_Defs = True
End Function

Private Function Tok_Def() As Boolean
    Dim VarName As String
    Dim Value   As String
    Dim ValType As TokenType
    
    If Match(CBR) Then _
        Exit Function _
    Else _
        Call Rebound
    
    If Not Tok_LHS(VarName) Then _
        Exit Function
    
    If Not Match(DEF) Then
        m_Error = "Expected ':'"
        Exit Function
    End If
    
    If Not Tok_RHS(ValType, Value) Then _
        Exit Function
    
    If Not AssignValue(VarName, Value) Then _
        Exit Function
    
    Tok_Def = True
End Function

Private Function Tok_LHS(ByRef Value As String) As Boolean
    If Not Match(ID) Then
        m_Error = "Expected ID"
        Exit Function
    End If
    
    Tok_LHS = True
    Value = TokenVal
End Function

Private Function Tok_RHS( _
  ByRef ValueType As TokenType, _
  ByRef Value As String _
) As Boolean
    Select Case m_Scanner.GetNextToken()
        Case ID
            ValueType = ID
        Case NUM
            ValueType = NUM
        Case STRVAL
            ValueType = STRVAL
        Case Else
            m_Error = "Expected ID, NUM or STRVAL"
            Exit Function
    End Select
    
    Tok_RHS = True
    Value = TokenVal
End Function
'----------------------------------------------------

Private Function AssignValue(Variable As String, Value As String) As Boolean
    With CurrentStyle
        Select Case LCase$(Variable)
            Case "color"
                .ForeColor = Str2Col(Value)
            Case "font-family"
                .Font.Name = Value
            Case "font-weight"
                Select Case LCase$(Value)
                    Case "bold"
                        .Font.Bold = True
                    Case "normal"
                        .Font.Bold = False
                    Case Else
                        m_Error = "Expected 'bold' or 'normal'"
                        Exit Function
                End Select
            Case "font-style"
                Select Case LCase$(Value)
                    Case "italic"
                        .Font.Italic = True
                    Case "normal"
                        .Font.Italic = False
                    Case Else
                        m_Error = "Expected 'italic' or 'normal'"
                        Exit Function
                End Select
            Case "font-underline"
                Select Case LCase$(Value)
                    Case "underline"
                        .Font.Underline = True
                    Case "normal"
                        .Font.Underline = False
                    Case Else
                        m_Error = "Expected 'underline' or 'normal'"
                        Exit Function
                End Select
            Case "font-strikethrough"
                Select Case LCase$(Value)
                    Case "strikethrough"
                        .Font.Strikethrough = True
                    Case "normal"
                        .Font.Strikethrough = False
                    Case Else
                        m_Error = "Expected 'strikethrough' or 'normal'"
                        Exit Function
                End Select
        End Select
    End With
    
    AssignValue = True
End Function
'----------------------------------------------------

Private Function Match(Token As TokenType) As Boolean
    Match = m_Scanner.GetNextToken() = Token
End Function

Private Sub Rebound()
    Call m_Scanner.Rebound
End Sub
